
.Dialog .close_dialog_button,
.Dialog .close_dialog_button:hover {
    position: absolute;
    right: 0.5em;
    top: 0.5em;
    border: none;
    padding: 0;
    margin: 0;
    cursor: pointer;
    background-color: transparent;
    background-image: none;
}

.Dialog .dialog_header {
    padding: 1.4em 0;
}

.Dialog .dialog_heading {
    margin: 0;
    color: #333;
    font-family: Arial;
    font-size: 1.5em;
}

