
.TranslationWorkflowPathEditor {
    display: inline-block;
}

.TranslationWorkflowPathEditor .source_language {
    display: inline-block;
}

.TranslationWorkflowPathEditor .target_language {
    display: inline-block;
    margin-left: 0.4em;
}

.TranslationWorkflowPathEditor select {
    margin-left: 0.4em;
}

