#-*- coding: utf-8 -*-
u"""

@author:		Martí Congost
@contact:		marti.congost@whads.com
@organization:	Whads/Accent SL
@since:			June 2008
"""
from .page import (
    Page as StandardPage,
    PageTranslation as StandardPageTranslation
)

