
.LoginBlockView .identity_providers_box {
    margin-bottom: 2em;
}

.IdentityProvidersBox .identity_provider_button {
    display: inline-block;
    border-radius: 0.3em;
    background-color: #555;
    border: 1px solid #333;
    font-size: 1em;
    box-shadow: 0 0 16px rgba(0,0,0,0.4);
}

.IdentityProvidersBox .identity_provider_button:hover {
    background-color: #444;
}

.IdentityProvidersBox .identity_provider_button + .identity_provider_button {
    margin-left: 0.5em;
}

.IdentityProvidersBox .identity_provider_icon {
    vertical-align: middle;
    display: inline-block;
    margin: 0 0.8em;
}

.IdentityProvidersBox .identity_provider_label {
    vertical-align: middle;
    display: inline-block;
    color: white;
    font-size: 1.4em;
    padding: 0.7em;
    border-left: 1px solid #333;
    text-align: center;
    text-decoration: none;
    width: 7em;
}

/* Google button */
.IdentityProvidersBox .identity_provider_button[data-woost-identity-provider='Google'] {
    background-color: #e0492f;
}

.IdentityProvidersBox .identity_provider_button[data-woost-identity-provider='Google']:hover {
    background-color: #c63117;
}

.IdentityProvidersBox .identity_provider_button[data-woost-identity-provider='Google'],
.IdentityProvidersBox .identity_provider_button[data-woost-identity-provider='Google'] .identity_provider_label {
    border-color: #c2341d;
}

/* Facebook button */
.IdentityProvidersBox .identity_provider_button[data-woost-identity-provider='Facebook'] {
    background-color: #395697;
}

.IdentityProvidersBox .identity_provider_button[data-woost-identity-provider='Facebook']:hover {
    background-color: #2b3c60;
}

.IdentityProvidersBox .identity_provider_button[data-woost-identity-provider='Facebook'],
.IdentityProvidersBox .identity_provider_button[data-woost-identity-provider='Facebook'] .identity_provider_label {
    border-color: #3b4865;
}

