
.EditBlocksView .languages_dropdown {
    float: right;
    padding-top: 0.5em;
}

.EditBlocksView .languages_dropdown > .panel_wrapper > .panel {
    left: auto;
    right: 0;
}

.EditBlocksView .EditBlocksSlotList .slot_header {
    font-size: 0.8em;
    color: #444;
}

.EditBlocksView .EditBlocksSlotList .slot_blocks {
    position: relative;
    padding: 1em;
    border: 1px dashed #888;
    background-color: white;
}

.EditBlocksView .EditBlocksSlotList .slot.folded > .slot_blocks {
    display: none;
}

.EditBlocksView > .body > .toolbar .close_action {
    float: right;
}

.EditBlocksView .EditBlocksSlotList .slot_blocks .BlockDisplay + .slot_toolbar {
    margin-top: 1em;
}

.EditBlocksView .EditBlocksSlotList .slot_blocks .slot_toolbar form {
    display: inline-block;
}

.EditBlocksView .EditBlocksSlotList .slot_blocks .slot_toolbar button {
    font-size: 0.8em;
    background: none;
    border: none;
    box-shadow: none;
    padding: 0;
    cursor: pointer;
    text-align: left;
    display: inline-block;
}

.EditBlocksView .EditBlocksSlotList .slot_blocks .slot_toolbar button img {
    position: relative;
    top: 0.2em;
}

.EditBlocksView .EditBlocksSlotList .slot_blocks .slot_toolbar > * + * {
    margin-left: 0.5em;
}

.EditBlocksView .EditBlocksSlotList .block_type {
    font-size: 0.9em;
}

.EditBlocksView .EditBlocksSlotList .block_title {
    color: #333;
}

/* Block picker dialog
-----------------------------------------------------------------------------*/
.block_picker_dialog {
    padding: 1em;
}

.EditBlocksSlotList-block_picker_dialog {
    width: auto;
    height: auto;
}

.block_picker_dialog .new_blocks_panel,
.block_picker_dialog .common_blocks_panel {
    display: inline-block;
    vertical-align: top;
}

.block_picker_dialog .new_blocks_panel .new_blocks_palette,
.block_picker_dialog .common_blocks_panel .common_blocks_form {
    max-height: 40em;
    overflow: auto;
    padding-right: 1em;
}

.block_picker_dialog .new_blocks_panel {
    width: 20em;
}

.block_picker_dialog.with_common_blocks .new_blocks_panel {
    padding-right: 1em;
    border-right: 1px dotted #ccc;
}

.block_picker_dialog .common_blocks_panel {
    padding-left: 1em;
}

.block_picker_dialog .panel_header {
    font-weight: bold;
    font-size: 1em;
    margin-bottom: 0.8em;
}

.block_picker_dialog form {
    margin: 0;
    padding: 0;
}


.block_picker_dialog .new_blocks_panel button,
.block_picker_dialog .common_blocks_panel button {
    display: inline-block;
    text-align: left;
    background: none;
    box-shadow: none;
    border: none;
    padding: 0;
    margin: 0;
    cursor: pointer;
}

.block_picker_dialog .new_blocks_panel .new_blocks_palette .palette_entry .type_label {
    font-size: 0.9em;
}

.block_picker_dialog .common_blocks_panel .common_block_entry + .common_block_entry {
    margin-top: 0.3em;
}

.block_picker_dialog .common_blocks_panel .ItemLabel {
    font-size: 0.9em;
}

.block_picker_dialog .new_blocks_panel .palette_entry img,
.block_picker_dialog .common_blocks_panel .ItemLabel img {
    position: relative;
    top: 0.2em;
}

.block_picker_dialog .common_blocks_panel .ItemLabel img {
    margin-right: 0.4em;
}

.block_picker_dialog .dialog_buttons {
    margin-top: 1em;
    text-align: right;
}

/* BlockDisplay
-----------------------------------------------------------------------------*/
.EditBlocksView .BlockDisplay {
    position: relative;
    border: 1px solid #aaa;
    border-radius: 0.3em;
    padding: 0.4em;
    padding-top: 0.3em;
    background-color: #ddd;
    box-shadow: 0 2px 3px #ccc;
    -moz-user-select: none;
    -khtml-user-select: none;
    -webkit-user-select: none;
    user-select: none;
    cursor: move;
}

.EditBlocksView .BlockDisplay:focus {
    outline: none;
    border-color: #666;
}

.EditBlocksView .BlockDisplay.disabled_block {
    background-color: #eedbdb;
    border-color: #cd9d9d;
}

.EditBlocksView .BlockDisplay + .BlockDisplay {
    margin-top: 1em;
}

.EditBlocksView .BlockDisplay > .status_labels {
    position: absolute;
    right: 0.5em;
    top: 0.5em;
}

.EditBlocksView .BlockDisplay > .status_labels .status_label {
    cursor: default;
}

.EditBlocksView .BlockDisplay > .status_labels .status_label + .status_label {
    margin-left: 0.8em;
}

.EditBlocksView .BlockDisplay > .status_labels .status_label .label {
    position: relative;
    top: -0.3em;
    font-size: 0.8em;
    font-style: italic;
    color: #444;
}

.EditBlocksView .BlockDisplay .block_handle > .button > .label {
    font-size: 0.9em;
}

.EditBlocksView .BlockDisplay .block_handle > .button > .label > * {
    vertical-align: middle;
}

.EditBlocksView .BlockDisplay .block_handle > .panel_wrapper > .panel {
    padding: 0;
}

.EditBlocksView .BlockDisplay .block_handle > .button > .label > .block_image {
    margin-right: 0.2em;
}

.EditBlocksView .BlockDisplay .slot_header {
    font-size: 0.7em;
    margin-top: 1.2em;
    margin-bottom: 0.4em;
    display: inline-block;
}

.EditBlocksView .EditBlocksSlotList.foldable .slot_header {
    cursor: pointer;
}

.EditBlocksView .EditBlocksSlotList.foldable .slot_header:before {
    content: "▼";
    display: inline-block;
    width: 1.1em;
}

.EditBlocksView .EditBlocksSlotList.foldable .slot.folded .slot_header:before {
    content: "▶";
    display: inline-block;
    width: 1.1em;
}

.EditBlocksView .BlockDisplay .block_handle .panel {
    width: 15em;
}

.EditBlocksView .BlockDisplay .block_menu button {
    display: block;
    box-sizing: border-box;
    width: 100%;
    text-align: left;
    background: none;
    box-shadow: none;
    border: none;
    padding: 0.4em 0.5em;
    margin: 0;
    cursor: pointer;
    font-size: 0.75em;
}

.EditBlocksView .BlockDisplay .block_menu button img {
    position: relative;
    top: 0.2em;
    margin-right: 0.2em;
}

.EditBlocksView .BlockDisplay .block_menu button:hover {
    background-color: #ddd;
}

.EditBlocksView .BlockDisplay .block_menu button:focus {
    border: 1px dotted #bbb;
    outline: none;
}

.EditBlocksView .BlockDisplay .content_preview {
    margin-top: 0.5em;
}

.EditBlocksView .BlockDisplay .content_preview:empty {
    display: none;
}

.EditBlocksView .BlockDisplay.cut,
.EditBlocksView .BlockDisplay.drag {
    opacity: 0.7;
}

/* TextBlockDisplay
-----------------------------------------------------------------------------*/
.EditBlocksView .VideoBlockDisplay .content_preview,
.EditBlocksView .TextBlockDisplay .content_preview,
.EditBlocksView .NewsletterContentDisplay .content_preview {
    display: table-row;
    border-spacing: 0.5em;
}

.EditBlocksView .TextBlockDisplay .text_preview {
    margin: -0.3em;
}

.EditBlocksView .TextBlockDisplay .translation_preview,
.EditBlocksView .NewsletterContentDisplay .translation_preview {
    display: table-cell;
    vertical-align: top;
    padding: 0.3em;
    border: 1px solid #bbb;
    border-radius: 0.3em;
    background-color: #e8e8e8;
}

.EditBlocksView .TextBlockDisplay .translation_content,
.EditBlocksView .NewsletterContentDisplay .translation_content {
    max-height: 12em;
    min-width: 15em;
    overflow: auto;
    font-size: 0.8em;
}

.EditBlocksView .TextBlockDisplay .translation_label,
.EditBlocksView .NewsletterContentDisplay .translation_label {
    font-size: 0.7em;
    font-weight: bold;
    color: #444;
    margin-bottom: 0.4em;
}

.EditBlocksView .TextBlockDisplay .translation_content p:first-child,
.EditBlocksView .NewsletterContentDisplay .translation_content p:first-child {
    margin-top: 0;
}

.EditBlocksView .TextBlockDisplay .translation_content p:last-child,
.EditBlocksView .NewsletterContentDisplay .translation_content p:last-child {
    margin-bottom: 0;
}

.EditBlocksView .VideoBlockDisplay .image_gallery,
.EditBlocksView .TextBlockDisplay .image_gallery {
    vertical-align: top;
    display: table-cell;
    padding-left: 1em;
    padding-top: 0.5em;
    min-width: 14em;
}

.EditBlocksView .VideoBlockDisplay .image_gallery li,
.EditBlocksView .TextBlockDisplay .image_gallery li {
    float: none;
    display: inline-block;
    margin: 0;
}

.EditBlocksView .VideoBlockDisplay .image_gallery li + li,
.EditBlocksView .TextBlockDisplay .image_gallery li + li {
    margin-left: 1em;
}

.EditBlocksView .VideoBlockDisplay .image_gallery img,
.EditBlocksView .TextBlockDisplay .image_gallery img {
    box-shadow: 0 2px 2px #bbb;
}

.EditBlocksView .insertion_marker {
    outline: 0.1em solid #44ddff;
    margin-top: 0.5em;
    margin-bottom: 1em;
}

.EditBlocksView .insertion_marker:first-child {
    margin: 0;
    position: relative;
    top: -0.5em;
}

/* PublishableListingDisplay
-----------------------------------------------------------------------------*/
.EditBlocksView .PublishableListingDisplay .publishable_listing ul {
    list-style-type: none;
    display: block;
    margin: 0;
    padding: 0;
    padding-left: 1.5em;
}

.EditBlocksView .PublishableListingDisplay .publishable_listing li {
    display: block;
    margin: 0;
    margin-bottom: 0.5em;
    padding: 0;
}

.EditBlocksView .PublishableListingDisplay .publishable_listing li a {
    text-decoration: none;
    font-size: 0.9em;
}

.EditBlocksView .PublishableListingDisplay .publishable_listing li a span {
    text-decoration: underline;
}

