
.NewsletterLayout,
.NewsletterLayout table {
    width: 100%;
}

.NewsletterLayout .newsletter_root_cell {
    text-align: center;
}

.NewsletterLayout .newsletter_root_cell td {
    text-align: left;
}

.NewsletterLayout .newsletter_blocks {
    width: 600px;
}

.NewsletterLayout img {
    display: block;
}

.NewsletterLayout td {
    vertical-align: top;
}

.NewsletterLayout td > *:first-child,
.NewsletterContentView .content_text > *:first-child {
    margin-top: 0;
}

.NewsletterLayout td > *:last-child,
.NewsletterContentView .content_text > *:last-child {
    margin-bottom: 0;
}

