
.TranslationWorkflowRequestCard .request_fields {
    margin-top: 1em;
}

.TranslationWorkflowRequestCard .request_field {
    margin-top: 0.5em;
}

.TranslationWorkflowRequestCard .request_field .field_label {
    display: inline-block;
    vertical-align: top;
    width: 12em;
    color: #333;
    font-size: 0.9em;
}

.TranslationWorkflowRequestCard .request_field .field_label:after {
    content: ':';
}

.TranslationWorkflowRequestCard .request_field .field_value {
    display: inline-block;
    vertical-align: top;
}

.TranslationWorkflowRequestCard .request_language_field .field_value {
    font-size: 0.9em;
}

