
.BackOfficeLayout .TranslationWorkflowTable,
.BackOfficeLayout .TranslationWorkflowTable tr,
.BackOfficeLayout .TranslationWorkflowTable th,
.BackOfficeLayout .TranslationWorkflowTable td,
.BackOfficeLayout .Table .TranslationWorkflowTable,
.BackOfficeLayout .Table .TranslationWorkflowTable tr,
.BackOfficeLayout .Table .TranslationWorkflowTable th,
.BackOfficeLayout .Table .TranslationWorkflowTable td {
    background-image: none;
    background-color: transparent;
    border: none;
}

.BackOfficeLayout .TranslationWorkflowTable {
    width: 100%;
    border-collapse: collapse;
}

.BackOfficeLayout .TranslationWorkflowTable th,
.BackOfficeLayout .Table .TranslationWorkflowTable th {
    font-weight: bold;
    color: #333;
    text-align: left;
}

.BackOfficeLayout .TranslationWorkflowTable .header_cell {
    width: 20%;
}

.BackOfficeLayout .TranslationWorkflowTable .source_cell,
.BackOfficeLayout .TranslationWorkflowTable .translation_cell {
    width: 40%;
}

.BackOfficeLayout .TranslationWorkflowTable .missing_translation_notice {
    color: #999;
    font-style: italic;
}

.BackOfficeLayout .Form .field_instance-TranslationWorkflowTable {
    float: none !important;
}

