
.BackOfficeLayout .ContentView .tab_state_sign {
    margin-right: 0.6em;
}

.BackOfficeEditView .translation_workflow_controls {
    padding-top: 0.6em;
    float: right;
    vertical-align: middle;
    margin-right: 2em;
}

.BackOfficeEditView .translation_workflow_controls label {
    font-size: 0.9em;
}

.BackOfficeEditView .translation_workflow_controls input[type=checkbox] {
    position: relative;
    top: 0.2em;
    margin: 0;
    margin-right: 0.3em;
}

.BackOfficeLayout .TranslationWorkflowRequestStateSign {
    margin-right: 0.8em;
}

.BackOfficeLayout .TranslationWorkflowStateSign {
    display: inline-block;
    vertical-align: middle;
    border: none !important;
    font-size: 0.7em;
    border-radius: 8px;
    line-height: 0;
    width: 8px;
    height: 8px;
    padding: 0;
}

.BackOfficeLayout .ContentTable .TranslationWorkflowRequestStateSign > .label > .text_wrapper {
    opacity: 0.7;
}

.BackOfficeLayout .ContentTable .TranslationWorkflowRequestStateSign[data-woost-context-menu='expanded'] > .label > .text_wrapper,
.BackOfficeLayout .ContentTable .TranslationWorkflowRequestStateSign:hover > .label > .text_wrapper {
    opacity: 1;
}

