
.TextBlockView {
    overflow: hidden;
}

.TextBlockView > .block_link {
    display: block;
}

.TextBlockView .image_gallery .image_entry {
    margin: 0;
    margin-right: 1em;
    margin-bottom: 1em;
}

.TextBlockView.single_image.no_text .image_gallery .image_entry {
    margin: 0;
}

/* Top left */
.TextBlockView.top_left .image_gallery .image_entry,
.TextBlockView.top_right .image_gallery .image_entry {
    display: inline-block;
    vertical-align: top;
}

.TextBlockView.top_left .image_gallery .image_entry > a {
    text-align: left;
}

/* Top right */
.TextBlockView.top_right .image_list {
    text-align: right;
}

.TextBlockView.top_right .image_gallery .image_entry > a {
    text-align: right;
}

/* Float top left */
.TextBlockView.float_top_left .image_gallery {
    float: left;
}

/* Float top right */
.TextBlockView.float_top_right .image_gallery {
    float: right;
}

.TextBlockView.float_top_right .image_gallery .image_entry {
    margin-right: 0;
    margin-left: 1em;
}

/* Column (both left and right) */
.TextBlockView .columns_wrapper {
    display: table-row;
}

.TextBlockView .columns_wrapper .image_gallery,
.TextBlockView .columns_wrapper .text_container {
    display: table-cell;
    vertical-align: top;
}

.TextBlockView.column_right .image_gallery .image_entry {
    margin-right: 0;
    margin-left: 1em;
}

/* Center top */
.TextBlockView.center_top .image_gallery > ul {
    text-align: center;
}

.TextBlockView.center_top .image_gallery > ul > li {
    display: inline-block;
}

/* Center bottom */
.TextBlockView.center_bottom .image_gallery > ul {
    text-align: center;
}

.TextBlockView.center_bottom .image_gallery > ul > li {
    vertical-align: top;
    display: inline-block;
}

/* Inline */
.TextBlockView.inline * {
    vertical-align: middle;
}

.TextBlockView.inline .text_container * {
    display: inline !important;
    margin: 0;
    padding: 0;
}

.TextBlockView.inline a.block_link {
    text-decoration: none;
}

.TextBlockView.inline a.block_link .text_container {
    text-decoration: underline;
}

/* Fallback */
.TextBlockView.fallback .text_container {
    position: absolute;
    left: -10000px;
    top: 0;
}

.TextBlockView.fallback .image_gallery li {
    margin: 0;
}

.TextBlockView.fallback .image_gallery li img {
    display: block;
}

/* PublishableGrid
-----------------------------------------------------------------------------*/
.PublishableGrid ul.publishable_list {
    padding: 0;
}

.PublishableGrid .entry {
    display: inline-block;
    vertical-align: top;
    text-align: center;
    padding: 1em;
    margin: 0;
    width: 20em;
}

.PublishableGrid li.entry > .ImageGallery {
    display: inline-block;
}

.PublishableGrid li.entry {
    list-style: none;
}

.PublishableGrid .entry .publishable_view {
    display: block;
    margin-bottom: 1em;
}

.PublishableGrid .entry .AudioPlayer  {
    width: 12em !important;
    margin-left: 4em;
}

.PublishableGrid .entry .AudioPlayer .mejs-controls > * {
    display: none;
}

.PublishableGrid .entry .AudioPlayer .mejs-controls > .mejs-playpause-button {
    display: block;
}

.PublishableGrid .entry .AudioPlayer .mejs-time-rail {
    display: block;
    width: 7.4em !important;
}

.PublishableGrid .entry .AudioPlayer .mejs-time-total {
    width: 7.4em !important;
}

.PublishableGrid .entry .AudioPlayer .mejs-time-loaded {
    background: none !important;
    background-color: #95889c !important;
    background-color: #777 !important;
}

/* PublishableCollage
-----------------------------------------------------------------------------*/
.scripted .PublishableCollage {
    visibility: hidden;
}

.scripted .PublishableCollage.layout_ready {
    visibility: visible;
}

.PublishableCollage .publishable_list {
    list-style-type: none;
    padding: 0;
    line-height: 0;
}

.PublishableCollage .entry {
    display: inline-block;
    margin: 0;
}

.PublishableCollage[data-cocktail-collage-layout=rows] .entry .ThumbnailLink,
.PublishableCollage[data-cocktail-collage-layout=rows] .entry .ThumbnailLink > .thumbnail {
    display: block;
}

/* PublishablePopUp
-----------------------------------------------------------------------------*/
.PublishablePopUp {
    cursor: pointer;
}

