#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.html import templates

SlideShowBlockView = templates.get_class("woost.views.SlideShowBlockView")

