
.TranslationWorkflowRequestForm .translation_source_display {
    display: inline-block;
    vertical-align: top;
    margin-right: 0.8em;
    background-color: #f5f5f5;
    border: 1px solid #ccc;
    padding: 0.15em 0.2em;
}

.TranslationWorkflowRequestForm .field_instance-TextArea .translation_source_display {
    overflow: auto;
}

.TranslationWorkflowRequestForm .field_instance-TextArea .translation_source_display,
.TranslationWorkflowRequestForm .translation_source_display + textarea {
    margin-top: 0.3em;
}

.TranslationWorkflowRequestForm .translation_source_display,
.TranslationWorkflowRequestForm .translation_source_display + textarea,
.TranslationWorkflowRequestForm .translation_source_display + input {
    width: 30em !important;
}

.TranslationWorkflowRequestForm .field_instance-TextArea .translation_source_display {
    height: 7em;
}

.TranslationWorkflowRequestForm .translation_source_display + textarea {
    height: 8.2em;
}

.TranslationWorkflowRequestForm fieldset.translation_request > .fieldset_fields > fieldset.translation_request-translated_values .field_instance {
    float: none;
}

.TranslationWorkflowRequestForm fieldset.translation_request > .fieldset_fields > fieldset.translation_request-info {
    width: 15%;
}

.TranslationWorkflowRequestForm fieldset.translation_request-changelog .field_instance {
    float: none !important;
}

