
.ChangeLog * {
    -webkit-box-sizing: border-box;
    -moz-box-sizing: border-box;
    box-sizing: border-box;
}

.ChangeLog ul {
    list-style-type: none;
    padding: 0;
    margin: 0;
}

.ChangeLog .changeset_list_entry + .changeset_list_entry {
    margin-top: 2em;
}

.ChangeLog .changeset_date_display,
.ChangeLog .changeset_author_display {
    display: inline-block;
    vertical-align: middle;
    font-size: 0.9em;
}

.ChangeLog .changeset_date_display {
    margin-right: 0.4em;
    color: #777;
}

.ChangeLog .changeset_author_display .text_wrapper {
    color: #777;
}

.ChangeLog .change_list {
    margin-top: 0.5em;
}

.ChangeLog .change_list_grouped_entry,
.ChangeLog .change_list_entry {
    display: table;
    width: 100%;
    background-color: #f0f0f0;
    border: 1px solid #e0e0e0;
}

.ChangeLog .change_info {
    padding: 0.4em;
    display: table-cell;
    vertical-align: top;
    width: 30%;
    background-color: #e5e5e5;
}

.ChangeLog .change_action_display,
.ChangeLog .change_target_display {
    display: inline-block;
    vertical-align: middle;
}

.ChangeLog .change_action_display {
    opacity: 0.5;
}

.ChangeLog .change_list_grouped_entry .change_action_display,
.ChangeLog [data-woost-ChangeLog-explicit='true'] .change_action_display {
    opacity: 1;
}

.ChangeLog [data-woost-ChangeLog-action='create'] .change_action_display {
    color: #060;
}

.ChangeLog [data-woost-ChangeLog-action='modify'] .change_action_display {
    color: #226;
}

.ChangeLog [data-woost-ChangeLog-action='delete'] .change_action_display {
    color: #600;
}

.ChangeLog .change_action_display {
    margin-right: 0.4em !important;
}

.ChangeLog .change_diff {
    display: table-cell;
    vertical-align: top;
    width: 70%;
}

