#-*- coding: utf-8 -*-
u"""

.. moduleauthor:: Martí Congost <marti.congost@whads.com>
"""
from cocktail.translations import translations

translations.define(
    "woost.extensions.googleanalytics."
    "default_custom_definitions.locale.title",
    ca = u"Woost: idioma",
    es = u"Woost: idioma",
    en = u"Woost: locale"
)

translations.define(
    "woost.extensions.googleanalytics."
    "default_custom_definitions.roles.title",
    ca = u"Woost: rols d'usuari",
    es = u"Woost: roles de usuario",
    en = u"Woost: user roles"
)

translations.define(
    "woost.extensions.googleanalytics."
    "default_custom_definitions.path.title",
    ca = u"Woost: camí",
    es = u"Woost: camino",
    en = u"Woost: path"
)

translations.define(
    "woost.extensions.googleanalytics."
    "default_custom_definitions.publishable.title",
    ca = u"Woost: publicable",
    es = u"Woost: publicable",
    en = u"Woost: publishable"
)

translations.define(
    "woost.extensions.googleanalytics."
    "default_custom_definitions.type.title",
    ca = u"Woost: tipus",
    es = u"Woost: tipo",
    en = u"Woost: type"
)

